#include <avr/io.h>			// Muss immer sein
#include <avr/pgmspace.h> 	// Fuer Flash
#include <inttypes.h> 		// Fuer Datentyp
#include <avr/eeprom.h>		// Fuer EEPROM
#include "main.h"			// Allgemeine Programmablauf
#include "indikator.h"		// Indikator bedienen
#include "semisegment.h"	// Indikatorschaltung
#include "knopf.h" 			// Tasten befragen

// Kodetabelle fuer Segm. Koden selbst in semisegment.h beschrieben 
static const char semiseg[12] PROGMEM = {ZIF_0,ZIF_1,ZIF_2,ZIF_3,
ZIF_4,ZIF_5,ZIF_6,ZIF_7,ZIF_8,ZIF_9,
ZIF_MINUS,ZIF_LEER};

#define ZIF_LEER_CODE 11 // Kode, um Ziffer auszuschalten
// Kodetabelle fuer Ziffer. Koden selbst in semisegment.h beschrieben 
static const char semiras[4] PROGMEM = {RASR_0,RASR_1,RASR_2,RASR_3};
static unsigned char rasr = 0; // Zaehler fuer Ziffer 
char zif_ind[4] = {0,0,0,0}; // Puffer fuer Ziffer 

// Fuer Verzoegerung
// Inline-Funktion
static inline void __attribute__((always_inline))
delayloop8 (unsigned char count)
{
	/* Die Schleife dauert  3 * count + 3  Ticks */
/* bei F_CPU 10 MHz max 76,8 mcs */
	asm volatile ("1:"           "\n\t"
		      "subi %A0,1" "\n\t"
		      "brcc 1b"
		      : "+d" (count));
}


void init_indikator(void){
	DDR_RASR |= (1<<RAS0)|(1<<RAS1)|(1<<RAS2)|(1<<RAS3);
	RAS_AUS;
	PSEG = 0;
	DDR_SEG = 0xff;	
	sym_ind(ZIF_LEER_CODE, ZIF_LEER_CODE, ZIF_LEER_CODE, ZIF_LEER_CODE);
	rasr = 0;
}


void sekdec(int a){ // Die Funktion wandelt A im Bereich 0-5999 
// nach BCD und gibt Ergebnis in Puffer zif_ind heraus.
// Verwendbar, um a auf Indikator auszugeben.
	for(int i=0;i<4;i++){
			zif_ind[i] = 0;
				}
	while(a>=600)
		{
			zif_ind[0]++;
			a-=600;
		}
	while(a>=60)
		{
			zif_ind[1]++;
			a-=60;
		}
	while(a>=10)
		{
			zif_ind[2]++;
			a-=10;
		}
	while(a>=1)
		{
			zif_ind[3]++;
			a-=1;
		} 
	// Null in hoechster Ziffer loeschen
		if (zif_ind[0] == 0) zif_ind[0] = ZIF_LEER_CODE;
}

void sym_ind(char a, char b, char c, char d){  // Bringt Kode a, b, c, d 
// in Indikatorpuffer. 1,2,3,4. Eingefuehrt, um Array zif_ind[4] nicht nach 
// aussen sichtbar zu machen.  

	zif_ind[0] = a;
	zif_ind[1] = b;
	zif_ind[2] = c;
	zif_ind[3] = d;
}

void ind(void){ // Bedienung fuer dynamische Indikation
// Wird gerufen, um Indikator zu regenerieren und Tasten zu entprellen.
	unsigned char x;
	PSEG = 0;	// Segmente ausschalten
	rasr++;		// Indizierte Ziffer aendern

// Wenn rasr = Zahl von Ziff. Vorbereitung fuer 
// Tastenabfrage in naechsten Indikator-Takt
	if(rasr == RASR_MAX) { // wenn Max, Vorbereitung fuer Tastenabfrage
		RAS_EIN;
		delayloop8(5); // Verzoegerung 1,8 us
		RAS_IN;
		}
		else { // wenn rasr != RASR_MAX
			// Wenn rasr > Zahl von Ziff., rasr = 0 und Tastenabfrage
			if(rasr > RASR_MAX) {
				rasr = 0;	//wenn Max+1, dann Null
				knopf_abfrage(); // Tasten entprellen,File: knopf.c
				RAS_AUS;
				RAS_OUT;
				}

		// weiter normale Indikation bei rasr = (0...max-1)


			x = zif_ind[rasr]; // Zif aus Ind-Puffer
			PSEG = pgm_read_byte (&semiseg[x]); // wandeln in 7-seg 
	         	// und in Seg-Port ausgeben
			  	// Doppelpunkt blinkt nur bei Zaehlen
				// Doppelpunkt verwalten
			if(!(FLAGG & (1<<EF_SEK))) PSEG |= (1<<SEG_H);
				else if((FLAGG & (1<<F_DP))) PSEG |= (1<<SEG_H);
					else PSEG &= ~(1<<SEG_H);

			PRASR |= pgm_read_byte (&semiras[rasr]); 
				// Nummer von Ziffer in kode vo Ziffer
				// und in Zif. Port 
		
	}
}
