#include <avr/io.h>			// Muss immer sein
#include <inttypes.h> 		// Fuer Datentyp
#include <avr/interrupt.h> 	// Fuer Interrupts
#include "knopf.h"		// Tasten befragen 
#include "main.h"		// Allgemeine Programmablauf

void init_knopf(void){

DDR_KN &= ~((1<<KN_START)|(1<<KN_U)); // Eingang
PORT_KN |= (1<<KN_START)|(1<<KN_U);	  // Pull-up

}

void knopf_abfrage(void){

	static unsigned char ct0 = 0xFF, ct1 = 0xFF, rpt;
	
	unsigned char i, key_pin;

// RAS_EIN, RAS_IN im vorigen Ind-Takt
// Gate-Kapazitaeten sind aufgeladen

	key_pin = PIN_KN;
	
	i = key_state ^ ~key_pin;     // key changed ?
	ct0 = ~( ct0 & i );           // reset or count ct0
	ct1 = ct0 ^ (ct1 & i);        // reset or count ct1
	i &= ct0 & ct1;               // count until roll over ?
	key_state ^= i;               // then toggle debounced state
	key_press |= key_state & i;   // 0->1: key press detect

	if( (key_state & REPEAT_MASK) == 0 )  // check repeat function
     rpt = REPEAT_START;                  // start delay
  	if( --rpt == 0 ){
    rpt = REPEAT_NEXT;                    // repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
}


// check if a key has been pressed. Each pressed key is reported
// only once
//
// Pruefung Taste gedruckt? 
unsigned char get_key_press( unsigned char key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_press;                          // read key(s)
  key_press ^= key_mask;                          // clear key(s)
  sei();
  return key_mask;
}

// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
// Pruefung lang gedruckt
unsigned char get_key_rpt( unsigned char key_mask )
{
  cli();                    // read and clear atomic !
  key_mask &= key_rpt;      // read key(s)
  key_rpt ^= key_mask;      // clear key(s)
  sei();
  return key_mask;
}
/* Um nun einen Stellwert hochzuzhlen, wird einfach die Funktion
get_key_rpt() aufgerufen, die dann das Tastenbit in key_rpt abfragt und
zuruecksetzt. Damit zwischen Abfragen und Zuruecksetzen kein
Timerinterrupt reinhaut, muss beides unter Interruptsperre erfolgen. */

// check if a key is pressed right now
//
// Ob Taste gehalten wird 
unsigned char get_key_state( unsigned char key_mask )

{
  key_mask &= key_state;
  return key_mask;
}

// Ob kurz gedruckt
unsigned char get_key_short( unsigned char key_mask )
{
	unsigned char x;
  	cli();  // read key state and key press atomic !
  	x = get_key_press( ~key_state & key_mask );
	sei();
  	return x;
}
/* Die naechste neue Funktion ist get_key_short() fuer die Erkennung des
kurz Drueckens. Das kann aber erst nach dem Loslassen erkannt werden.
Dazu wird der key_state ausgewertet, d.h. erst wenn dort das Tastenbit
wieder 0 ist, wird get_key_press() ein Wert uebergeben, und kann
ausgefuehrt werden. Damit dazwischen kein Timerinterrupt erfolgt, muss
vor der Abfrage von key_state der Interrupt gesperrt werden,
get_key_press() enabled dann wieder die Interrupts.
Wird nur get_key_short() aufgerufen, d.h. nicht mit get_key_long() im
Wechsel, dann liefert es auch einen Wert zurueck nach langem Druecken. */
//
unsigned char get_key_long( unsigned char key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}
/* get_key_long() wiederum liefert erst dann einen Wert, wenn beide
Funktionen get_key_rpt() und get_key_press() das Tastenbit gesetzt
haben. */

/* Funktionsweise */
/* Der Code basiert auf 8 parallelen vertikalen Zaehlern, 
die ueber die Variablen ct0 und ct1 aufgebaut werden */
/* 8 vertikale Zaehler in 2 8-Bit Variablen */
/* wobei jeweils ein Bit in ct0 mit dem gleichwertigen Bit 
in ct1 zusammengenommen einen 2-Bit-Zaehler bildet. Der Code 
der sich um die 8 Zaehler kuemmert, ist so geschrieben, dass er 
alle 8 Zaehler gemeinsam parallel behandelt. */
//  i = key_state ^ ~KEY_PIN;               // key changed ?
/* i enthaelt an dieser Stelle fuer jede Taste, die sich im Vergleich 
mit dem vorhergehenden entprellten Zustand (keystate) veraendert hat, 
ein 1 Bit. */
// ct0 = ~( ct0 & i );                     // reset or count ct0
// ct1 = ct0 ^ (ct1 & i);                  // reset or count ct1
/* Diese beiden Anweisungen erniedrigen den 2-Bit Zaehler ct0/ct1 
fuer jedes Bit um 1, welches in i gesetzt ist. Liegt an der 
entsprechenden Stelle in i ein 0 Bit vor (keine Aenderung des Zustands), 
so wird der Zaehler ct0/ct1 fuer dieses Bit auf 1 gesetzt. 
Der Grundzustand des Zaehlers ist als ct0 == 1 und ct1 == 1 (Wert 3). 
Der Zaehler zaehlt daher mit jedem ISR Aufruf, bei dem die Taste 
im Vergleich zu keystate als veraendert erkannt wurde */
//   ct1   ct0
//    1    1   // 3
//    1    0   // 2
//    0    1   // 1
//    0    0   // 0
//    1    1   // 3
// i &= ct0 & ct1;       // count until roll over ?
/* in i bleibt nur dort ein 1-Bit erhalten, wo sowohl in ct1 als auch 
in ct0 ein 1 Bit vorgefunden wird, der betreffende Zaehler also bis 3 
zaehlen konnte. Durch die zusaetzliche Verundung mit i wird der Fall 
abgefangen, dass ein konstanter Zaehlerwert von 3 in i ein 1 Bit 
hinterlaesst. Im Endergebnis bedeutet dass, dass nur ein Zaehlerwechsel 
von 0 auf 3 zu einem 1 Bit an der betreffenden Stelle in i fuehrt, 
aber auch nur dann, wenn in i an dieser Bitposition ebenfalls 
ein 1 Bit war (welches wiederrum deswegen auf 1 war, weil an diesem 
Eingabeport eine Veraenderung zum letzten bekannten entprellten Zustand 
festgestellt wurde). Alles zusammengenommen heisst das, dass ein 
Tastendruck dann erkannt wird, wenn die Taste 4 mal hintereinander 
in einem anderen Zustand vorgefunden wurde als dem zuletzt bekannten 
entprellten Tastenzustand. */
/* An dieser Stelle ist i daher ein Vektor von 8 Bits, von denen 
jedes einzelne der Bits darueber Auskunft gibt, ob die entsprechende 
Taste mehrmals hintereinander im selben Zustand angetroffen wurde, 
der nicht mit dem zuletzt bekannten Tastenzustand uebereinstimmt. 
Ist das der Fall, dann wird eine entsprechende Veraenderung des 
Tastenzustands in key_state registriert */
// key_state ^= i;  // then toggle debounced state
/* und wenn sich in key_state das entsprechende Bit von 0 auf 1 
veraendert hat, wird dieses Ereignis als 'Taste wurde niedergedrueckt' 
gewertet. */
// key_press |= key_state & i;  // 0->1: key press detect
/* Damit ist der Tasteneingang entprellt. Und zwar sowohl beim Druecken 
einer Taste als auch beim Loslassen (damit Tastenpreller beim Loslassen 
nicht mit dem Niederdruecken einer Taste verwechselt werden). 
Der weitere Code beschaeftigt sich dann nur noch damit, diesen entprellten 
Tastenzustand weiter zu verarbeiten. */
/* Der Codeteil sieht durch die Verwendung der vielen bitweisen 
Operationen relativ komplex aus. Behaelt man aber im Hinterkopf, dass 
einige der bitweisen wie ein 'paralles If' gleichzeitig auf allen 8 Bits 
eingesetzt werden, dann vereinfacht sich vieles. Ein */
// key_press |= key_state & i;
/* ist nichts anderes als ein */
	  // teste ob Bit 0 sowohl in key_state als auch in i gesetzt ist
      // und setze Bit 0 in key_press, wenn das der Fall ist
// if( ( key_state & ( 1 << 0 ) ) && ( i & ( 1 << 0 ) )
//       key_press |= ( 1 << 0 );

// Bit 1
//    if( ( key_state & ( 1 << 1 ) ) && ( i & ( 1 << 1 ) )
//       key_press |= ( 1 << 1 );

// Bit 2
//    if( ( key_state & ( 1 << 2 ) ) && ( i & ( 1 << 2 ) )
//       key_press |= ( 1 << 2 );

 //   ...

/* nur als wesentlich kompaktere Operation ausgefuehrt und fuer alle 
8 Bits gleichzeitig. Die Kuerze und Effizienz dieser paar Codezeilen 
ergibt sich aus dem Umstand, dass jedes Bit in den Variablen fuer 
eine Taste steht und alle 8 (maximal moeglichen) Tasten gleichzeitig 
die Operationen durchlaufen. */
/* Funktionsweisen der verschienden Modi anhand von Zeitstrahlen 
erklrt: https://www.mikrocontroller.net/topic/48465?goto=1753367#1844458 */
/* "Walkthrough" der verschiedenen Zustnder der Einzelnen Variablen 
anhand eines Tastendrucks (avrfreaks.net) 
http://www.avrfreaks.net/comment/726676#comment-726676 */

