#include <avr/io.h>			// Muss immer sein
#include <avr/interrupt.h> 	// Fuer Interrupts
#include <inttypes.h> 		// Fuer Datentyp
#include <avr/eeprom.h>		// Fuer EEPROM
#include "main.h"		// Allgemeine Programmablauf
#include "logik.h" 		// Logik
#include "time.h"		// Zeitfunktionen
#include "indikator.h"	// Bedienung von Indikator 
#include "semisegment.h"	// Indikatorschaltung
#include "knopf.h"		// Tastenabfrage

uint16_t voreinstellung EEMEM = 180; // Voreinstellung in EEPROM
    // Zahl von Sekunden

/**********************************************************/
/* Funktionen fuer logik **********************************/
void start_exposition(unsigned int laenge, unsigned int ton){
	time = timepres;	// Zeitvoreinstellung in Zahlregister
	sekdel_null(); // Sek. Divider > 0  File: time.c 
	dp_null();     // Divider > Null und DP > 1  File: time.c 
	start_exp_fort(laenge,ton);

}

// Unterbrochene Zahlen fortsetzen
void start_exp_fort(unsigned int laenge, unsigned int ton){
	sekdec(time);	// Ind. Puffer < Zahlregister File: indikator.c 
	FLAGG |= (1<<EF_SEK); // Flag fuer Sekundenzaehlen einsetzen
	UV_PORT |= (1<<UV_OUT); // UV-Strahler einschalten
	schall_ein(laenge,ton); // Schall File: time.c 
}

void stop_exposition(unsigned int laenge, unsigned int ton){
	UV_PORT &= ~(1<<UV_OUT); // UV-Strahler ausschalten
	FLAGG &= ~(1<<EF_SEK);	 // Sek. nicht zaehlen
	schall_ein(laenge, ton);	// Schall File: time.c 
	sekdec(timepres);		// Ind. < Zeitvoreinstellung File: indikator.c 

}

void lese_voreinstellung(void){
			timepres = eeprom_read_word(&voreinstellung);  // Lesen aus EEPRO
			sekdec(timepres);
}


void tasten_controlle(void){ // U O M SP START LICHT
	// Prueft Tastenzustand nach Entprellen
	 // und macht entsprechende Aktionen

	if(get_key_press(1<<KN_O)){
		if(!get_key_state(1<<KN_M)) timepres++;
			else timepres +=60;
		if(timepres>5999) timepres -=6000;
		schall_ein(20,K_ZV_E4);				// Schall File: time.c 
		sekdec(timepres);	// Ind. < Zeitvoreinstellung File: indikator.c 
	}

	if(get_key_press(1<<KN_U)){
		if(!get_key_state(1<<KN_M)) timepres--;
			else timepres -=60;
		if(timepres<0) timepres +=6000;
		schall_ein(20,K_ZV_E4);			// Schall File: time.c 
		sekdec(timepres);	// Ind. < Zeitvoreinstellung File: indikator.c 
	}

	if(get_key_rpt(1<<KN_O)){ // Lang gedruckte Taste, schnell vergroessern
		if(!get_key_state(1<<KN_M)) timepres++;
			else timepres +=60;
		if(timepres>5999) timepres -=6000;
		schall_ein(20,K_ZV_E4);			// Schall File: time.c 
		sekdec(timepres);	// Ind. < Zeitvoreinstellung File: indikator.c 
	}

	if(get_key_rpt(1<<KN_U)){ // Lang gedruckte Taste, schnell verkleinern
		if(!get_key_state(1<<KN_M)) timepres--;
			else timepres -=60;
		if(timepres<0) timepres +=6000;
		schall_ein(20,K_ZV_E4);			// Schall File: time.c 
		sekdec(timepres);		// Ind. < Zeitvoreinstellung File: indikator.c 
	}
	
	
	if(get_key_press(1<<KN_START)){

		if(!get_key_state(1<<KN_M)){ // wenn M nicht gedruckt

			if(FLAGG & (1<<EF_SEK)) stop_exposition(150,K_ZV_C4);
				else start_exposition(150,K_ZV_C4);	
				}
		  else { // wenn M gedruckt
		    if(FLAGG & (1<<EF_SEK)) stop_exposition(150,K_ZV_C4);
				else start_exp_fort(150,K_ZV_C4);	
		  }
	}

	if(get_key_press(1<<KN_SP)){
		if(!get_key_state(1<<KN_M)) {
			lese_voreinstellung();
			}		                   
			else eeprom_write_word(&voreinstellung, timepres); 
			// schreiben in EEPRO
	}

	if(get_key_press(1<<KN_LICHT)){
		FLAGG ^= (1<<F_LICHT); // Flag Beleuchtung invertieren
		LICHT_PORT ^= (1<<LICHT_OUT); // Port beleuchtung invertieren
		schall_ein(20,K_ZV_E4);				// Schall File: time.c 
	}
}


