/* Programm fuer Timer 2017_05_28_Timer */
/* Timer schaltet UV-Belichter fuer die Zeit bis 99 min. 59 sek.
Voreinstellung und Zaehlen (waehrend der Exposition) werden
auf dem LED-Indikator mit Doppelpunkt gezeigt.
Zusaetzlich lann man Hilfsbeleuchtung vom Arbeitsfeld einschalten */

// Quarz 10 MHz.
// Tasten 5=LICHT 1=U 2=O 3=M 4=SP 0=START 

#define F_CPU 10000000UL		// CPU_F fuer Verzoegerungen

#include <avr/io.h>			// Muss immer sein
#include <avr/interrupt.h> 	// Fuer Interrupts
#include <inttypes.h> 		// Fuer Datentyp
#include <avr/eeprom.h>		// Fuer EEPROM

#include "main.h" // Allgemeine Programmablauf 
#include "init.h" // Initialisieren 
#include "time.h" // Zeitfunktionen 
#include "indikator.h" // Indikator bedienen 
#include "knopf.h" // Tasten befragen 
#include "logik.h" // Logik 


// Globale Variablen
int time = 0; // Zeitregister, Zaehler der Zeit fuer UV-Timer
int timepres = 0; // Zeitvoreinstellung fuer UV-Timer
unsigned int schall_laenge; // Dauer des Klangs in ms
unsigned char key_state; // debounced and inverted key state:
                  // bit = 1: key pressed
// "fertige" Tastenzustand, nach Entprellen
unsigned char key_press; // key press detect
// Feststellen, ob Taste gedruckt wurde
unsigned char key_rpt;  // key long press and repeat
// Feststellen, ob Taste lange gedruckt wurde
// U O M SP START LICHT

// -- Interrupts -- 

// ISR_NAKED, da kein Register nebutzt wird, somit PUSH und POP ersparen
ISR(TIMER0_COMPA_vect, ISR_NAKED){ // Timer 0 interrupt.

	FLAGG |= (1<<F_PULS);  // Flag fuer Puls einsetzen
	reti(); // da ISR_NAKED
}

// Hauptprogramm

int main(void){
init();	// Initialisieren alles, File: init.c
// sekdec(timepres); Steht schon in init() - lese_voreinstellung()	

while(1){
// Tik bedienen, Indikator, Tasten und Time bedienen
if(FLAGG & (1<<F_PULS)){
	FLAGG &= ~(1<<F_PULS);  // Flag fuer Puls = 0

time_puls();
// Systemsynchr.: Puls fuer Indikator, Tastatur, Sekundenpuls. 
// Wird 1250 mal pro Sek. gerufen. File time.c

flagg_controlle();
// Pruefung auf Ereignisse und Reaktion Flaggs befragen, 
// die nach Timer 0 eingesetzt wurden. File: time.c 

tasten_controlle();
// 5=LICHT 1=U 2=O 3=M 4=SP 0=START
// Frueft Tastenzustand (nach Entprellen) und macht entsprechende 
// Handlungen File: logik.c


}
}
return 0;
}

