/* Alles, was Zeit und Systemtakt betrifft */

#include <avr/io.h>			// Muss immer sein
#include <inttypes.h> 		// Fuer Datentyp
#include "main.h"		// Allgemeine Programmablauf 
#include "time.h"		// Zeitfunktionen 
#include "indikator.h"	// Indikator bedienen 
#include "logik.h" 		// Logik 
#include "knopf.h"		// Tasten befragen 

static char time_indikator; // Fuer Indikator
static int time_sekunden;	// Fuer Sekunden 
static int time_doppelpunkt;	// Fuer Doppelpunkt 

void time_puls(void){ // Systemsynchr. Puls fuer Indikator,
						//Tasten, Sekunden.  1250 Hz 
  // Setzt Flags fuer Ereignisse ( Indikator bedienen, Sekunden zaehlen,
  // Doppelpunkt blinken )

	time_indikator--;  // Indikator-Divider
	if(!time_indikator) {
			time_indikator = T_IND;
			FLAGG |= (1<<F_IND);  // Flag fuer Indikator einsetzen
	}

	if(FLAGG & (1<<EF_SEK)){ // Nur wenn Zaehlen von Sekunden erlaubt
		time_sekunden--;  // Sekunden-Divider
		if(!time_sekunden){
				time_sekunden = T_SEK;
				FLAGG |= (1<<F_SEK); // Sekunden-Flag einstellen
				}
		time_doppelpunkt--;  // Doppelpunkt-Divider
		if(!time_doppelpunkt){
				time_doppelpunkt = T_DP;
				FLAGG ^= (1<<F_DP); // Flag Doppelpunkt invertieren
				}
	}

	if(schall_laenge){   // Schalllaenge bedienen
		schall_laenge--;
		if(!schall_laenge){
			TCCR1B = (1<<WGM12); // Schall aus
			ZV_PORT &= ~(1<<ZV_OUT); // = 0, um Verbrauch zu senken
		}
	}
}

void init_time(){  // Zeit initialisieren
	sekdel_null();
	time_indikator = T_IND;
	dp_null();	
	FLAGG = 0;
}

void sekdel_null(){ // Sekunden-Divider = 0
	time_sekunden = T_SEK;
}

void dp_null(){  // Doppelpunkt-Divider = 0 und DP = 1
	time_doppelpunkt = T_DP;
	FLAGG |= (1<<F_DP);
}

void sek_decrement(){ // Sekundenzaehler decrement. Wenn 0,UV aus und EF_SEK = 0
	time--;
	sekdec(time); // File: indikator.c
	if(time == 0){
		stop_exposition(1000, K_ZV_C4); // File: logik.c
	}
}

// Schall einschalten
void schall_ein(unsigned int laenge, unsigned int freq){ // Schalldauer in ms; 
// freq = K_Frequenz
	ZV_PORT &= ~(1<<ZV_OUT); // Schallausgang in 0
	
	if(freq){ // Wenn freq != 0
	ZV_DDR |= (1<<ZV_OUT); // Schallout als Ausgang
	OCR1A = freq;			// In Timer K_Frequenz.
	TCCR1B = (1<<WGM12)|(1<<CS10);	// Timer als CTC, prescaler 1
	schall_laenge = laenge + (laenge>>2); // Dauer Umrechnen, da Puls 1250 
	}
	else{ // Wenn freq = 0  - eine Variante fuer Pausen in Schall
	ZV_DDR &= ~(1<<ZV_OUT); // Schallout als Ausgang
	TCCR1B = (1<<WGM12);
	schall_laenge = laenge + (laenge>>2); // Dauer Umrechnen, da Puls 1250 
	}
}

void flagg_controlle(void){ // Pruefung auf Ereignisse und Reaktion
// Flaggs befragen, die nach Timer 0 eingesetzt wurden
	if(FLAGG & (1<<F_IND)) {  // Wenn Flag von Indikator
		FLAGG &= ~(1<<F_IND); // Flag in 0
		ind();                // Indikator bedienen File: indikator.c
		}

	if(FLAGG & (1<<F_SEK)) { // Wenn Flag von Sekunde
		FLAGG &= ~(1<<F_SEK);  // Flag in 0
		if(FLAGG & (1<<EF_SEK)) { // Wenn Flag fuer Sekundenzaehlen
			sek_decrement(); // Sekundenzaehler decrement File: time.c
			}
		}

}
